#if !defined HAVE_COMB_ALL_H__
#define      HAVE_COMB_ALL_H__
// This file is part of the FXT library.
// Copyright (C) 2010, 2011, 2012 Joerg Arndt
// License: GNU General Public License version 3 or later,
// see the file COPYING.txt in the main directory.


//: Combinatorics

//: subsets:
#include "comb/subset-gray.h"
#include "comb/subset-gray-delta.h"
#include "comb/ruler-func.h"
#include "comb/subset-monotone.h"
#include "comb/subset-lex.h"
#include "comb/subset-deltalex.h"
#include "comb/subset-debruijn.h"
#include "comb/ksubset-gray.h"
#include "comb/ksubset-twoclose.h"
#include "comb/ksubset-rec.h"
#include "comb/binary-sl-gray.h"

//: combinations:
#include "comb/combination-lex.h"
#include "comb/combination-colex.h"
#include "comb/combination-revdoor.h"
#include "comb/combination-mod.h"
#include "comb/combination-emk.h"
#include "comb/combination-enup.h"
#include "comb/combination-endo.h"
#include "comb/combination-chase.h"
#include "comb/combination-rec.h"
#include "comb/combination-pref.h"

//: compositions:
#include "comb/composition-colex.h"
#include "comb/composition-colex2.h"
#include "comb/composition-ex-colex.h"
#include "comb/composition-nz.h"
#include "comb/num-compositions.h"
#include "comb/composition-rank.h"
#include "comb/comp2comb.h"

//: parenthesis strings (Catalan objects) and their RGS:
#include "comb/paren.h"
#include "comb/paren-gray.h"
#include "comb/paren-pref.h"
#include "comb/catalan.h"
#include "comb/catalan-lex.h"
#include "comb/catalan-gray.h"
#include "comb/catalan-subset-lex.h"
#include "comb/catalan-gslex.h"
#include "comb/paren-string-to-rgs.h"

//: Motzkin paths:
#include "comb/is-motzkin-path.h"
#include "comb/motzkin-path-lex.h"


//: Dyck words and their RGS:
#include "comb/dyck-rgs.h"
#include "comb/dyck-gray.h"
#include "comb/dyck-gray2.h"
#include "comb/dyck-pref.h"
#include "comb/dyck-pref2.h"
#include "comb/dyck-rgs-subset-lex.h"

//: integer partitions:
#include "comb/partition-gen.h"
#include "comb/partition.h"
#include "comb/mpartition.h"

//: set partitions and their RGS:
#include "comb/setpart.h"
#include "comb/setpart-rgs-lex.h"
#include "comb/setpart-rgs-gray.h"
#include "comb/setpart-p-rgs-lex.h"
#include "comb/setpart-rgs-subset-lex.h"
#include "comb/rgs-maxincr.h"
#include "comb/rgs-fincr.h"
#include "comb/rgs-kincr.h"
#include "comb/check-setpart-rgs.h"

//: necklaces and deBruijn sequences:
#include "comb/necklace.h"
#include "comb/binary-necklace.h"
#include "comb/num-necklaces.h"
#include "comb/debruijn.h"
#include "comb/binary-debruijn.h"

//: mixed radix numbers:
#include "comb/mixedradix.h"
#include "comb/mixedradix-lex.h"
#include "comb/mixedradix-subset-lex.h"
#include "comb/mixedradix-sl-gray.h"
#include "comb/mixedradix-gslex.h"
#include "comb/mixedradix-gslex-alt.h"
#include "comb/mixedradix-gslex-alt2.h"
#include "comb/mixedradix-gray.h"
#include "comb/mixedradix-gray2.h"
#include "comb/mixedradix-modular-gray.h"
#include "comb/mixedradix-modular-gray2.h"
#include "comb/mixedradix-endo.h"
#include "comb/mixedradix-endo-gray.h"
#include "comb/mixedradix-sod-lex.h"
#include "comb/mixedradix-naf.h"
#include "comb/mixedradix-naf-gray.h"
#include "comb/mixedradix-naf-sl.h"

//: permutations:
#include "comb/perm-lex.h"
#include "comb/perm-lex-inv.h"
#include "comb/perm-lex2.h"
#include "comb/perm-colex.h"
#include "comb/perm-rec.h"
#include "comb/perm-mv0.h"
#include "comb/perm-ives.h"
#include "comb/perm-trotter.h"
#include "comb/perm-trotter-lg.h"
#include "comb/perm-gray-ffact.h"
#include "comb/perm-gray-ffact2.h"
#include "comb/perm-gray-rfact.h"
#include "comb/perm-gray-rot1.h"
#include "comb/perm-gray-lipski.h"
#include "comb/perm-gray-wells.h"
#include "comb/perm-heap.h"
#include "comb/perm-heap2.h"
#include "comb/perm-heap2-swaps.h"
#include "comb/perm-rot.h"
#include "comb/perm-pref.h"
#include "comb/perm-star.h"
#include "comb/perm-star-swaps.h"
#include "comb/perm-restrpref.h"
#include "comb/perm-rev.h"
#include "comb/perm-rev2.h"
#include "comb/perm-st.h"
#include "comb/perm-st-pref.h"
#include "comb/perm-st-gray.h"
#include "comb/perm-derange.h"
#include "comb/perm-involution.h"

#include "comb/cyclic-perm.h"
#include "comb/fact2num2perm.h"
#include "comb/fact2perm.h"
#include "comb/big-fact2perm.h"
#include "comb/num2perm.h"
#include "comb/fact2num.h"

//: k-permutations:
#include "comb/kperm-lex.h"
#include "comb/kperm-gray.h"

//: multiset permutations:
#include "comb/mset-perm-2invtab.h"
#include "comb/mset-perm-lex.h"
#include "comb/mset-perm-lex-rec.h"
#include "comb/mset-perm-pref.h"
#include "comb/mset-perm-gray.h"

//: n-dimensional Hilbert curve:
#include "comb/hilbert-ndim.h"
#include "comb/hilbert-ndim-rec.h"

// auxiliary routines:
#include "comb/comb-print.h"
#include "comb/endo-enup.h"

#include "comb/check-permgen.h"
#include "comb/check-kpermgen.h"
#include "comb/check-mixedradix.h"


#include "comb/test-gray.h"
#include "comb/delta2gray.h"
#include "comb/monotonic-gray.h"
#include "comb/gray-cycle-leaders.h"
#include "comb/acgray.h"

#include "comb/stringsubst.h"

#include "comb/fibonacci.h"





#endif  // !defined HAVE_COMB_ALL_H__
